
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <io.h>
#include <string.h>

/* Prototypes */
unsigned int LitFichier(char *fichier,char *buf_dest,unsigned int buf_taille);
void         EcritFichier(char *fichier,char *buf_src,unsigned int buf_taille);
void         GenErreur(char *message,unsigned char code);
void         cpstr(char *destination, char *source);
void         genheader(char *buffer,unsigned int taille,char *nom_prog);
void         CreeFichier(char *fichier);
void         TransBMP(unsigned char *Source);
unsigned int GetComment(unsigned char *comment);

/* Une petite union pour ne pas s'emmerder avec des masques et des dcalages*/
union nb
{
 char valc[2];
 short vals;
};



/* Lit un fichier binaire */
unsigned int LitFichier(char *fichier,char *buf_dest,unsigned int buf_taille)
{
  unsigned int Taille;
  int handle;

  if ((handle=open(fichier,O_RDONLY|O_BINARY))==-1)
    GenErreur("Error on file opening",1);

 /* Lecture du fichier */
 Taille=read(handle,buf_dest,buf_taille);

 if (Taille==-1)
   GenErreur("Error on file reading",1);

 close(handle);

 return Taille;
}

/* Ecrit un fichier binaire */
void EcritFichier(char *fichier,char *buf_src,unsigned int buf_taille)
{
  FILE *handle;

  if ((handle=fopen(fichier,"wb+"))==NULL)
    GenErreur("Error on file creating",1);

 /* Ecriture du fichier */
 if (fwrite(buf_src,sizeof(char),buf_taille,handle)==-1)
  GenErreur("Error on file writing",1);

 fclose(handle);
}


/* Gnre une erreur */
void GenErreur(char *message,unsigned char code)
{
   printf("%s.\n",message);
   exit(code);
}

/* Copie la chaine <source> vers <destination> */
void cpstr(char *destination, char *source)
{
 while(*destination++=*source++);
}


/* Gnre un header 85S */
void genheader(char *buffer,unsigned int taille,char *nom_prog)
{
 unsigned int i;
 union nb nombre;

 /* Header 1 */
 cpstr(buffer,"**TI85**"); buffer+=8;
 *buffer++=0x1A;
 *buffer++=0x0C;
 *buffer++=0;

 /* Commentaire */
 cpstr(buffer,"ImageView file. By Kaid [kaid@excite.fr]  "); buffer+=42;

 /* Header 2 */
 nombre.vals=taille+strlen(nom_prog)+10;
 *buffer++=nombre.valc[0]; *buffer++=nombre.valc[1];

 nombre.vals=strlen(nom_prog)+4;
 *buffer++=nombre.valc[0]; *buffer++=nombre.valc[1];

 nombre.vals=taille+2;
 *buffer++=nombre.valc[0]; *buffer++=nombre.valc[1];

 *buffer++=0x0C;        /* Programme non ditable */

 /* Copie du nom du programme */
 *buffer++=strlen(nom_prog);
 cpstr(buffer,nom_prog); buffer+=strlen(nom_prog);

 /* Header 3 */
 *buffer++=nombre.valc[0]; *buffer++=nombre.valc[1];

 nombre.vals=taille;
 *buffer++=nombre.valc[0]; *buffer++=nombre.valc[1];

 /* Permet de reconnaitre les fichiers produits par IVC */
 *buffer++=5;
 *buffer++=0;
}


/* Cre le fichier .85S  crire */
void CreeFichier(char fichier[])
{

 unsigned char Destination[2048],Source[1152];
 unsigned char comment[42];
 unsigned char nom_fich[16];

 unsigned short checksum=0;
 unsigned int i,j,TailleImg;

 /* On lit le fichier d'entre */
 TailleImg=LitFichier(fichier,Source,sizeof(Source));

 /* Met dans 'nom_fich' le nom du fichier */
 i=0;
 while(*fichier!='.' && *fichier)
  nom_fich[i++]=*fichier++;
 nom_fich[i]=0;



                        /* On teste le type de fichier */
 if (strcasecmp(fichier,".85I")==0)
 {
   printf("o Format: 85I\n");
   /* i est la taille du nom du fichier */
   i=Source[0x3C];
   /* Prend le commentaire */
   j=GetComment(comment);
   /* On compresse l'image */
   TailleImg=CompZCP(Source+0x41+i,Destination+0x43+j+strlen(nom_fich),1008);
 }
 else
  if (strcasecmp(fichier,".BMP")==0)
  {
    printf("o Format: BMP\n");
    /* Prend le commentaire */
    j=GetComment(comment);
    /* Transforme le BMP */
    TransBMP(Source);
    /* On compresse l'image */
    TailleImg=CompZCP(Source+0x3E,Destination+0x43+j+strlen(nom_fich),1024);
  }
  else
   GenErreur("Invalid format",9);

 /* Taille des donnes (Identificateur+Commentaire+Taille de l'image) */
 i=2+j+TailleImg;

 /* Gnre le Header*/
 genheader(Destination,i,nom_fich);

 /* Copie le commentaire */
 cpstr(Destination+0x43+strlen(nom_fich),comment);

 /* Calcule le checksum*/
 for(j=0x37;j<i+0x41+strlen(nom_fich);j++)
  checksum+=Destination[j];

 /* Et l'crit */
 Destination[j++]=checksum&0xFF;
 Destination[j++]=checksum>>8;


 strcat(nom_fich,".85S");
 EcritFichier(nom_fich,Destination,0x43+i+strlen(nom_fich));

 printf("o %s written (%d bytes)\n",nom_fich,i);
}


/* Transforme le BMP (Rotation 180+Mirroir+Negatif) */
void TransBMP(unsigned char *Source)
{
 unsigned int i,j;
 unsigned char c;

 for(i=0;i<32;i++)
 {
  for(j=0;j<16;j++)
  {
   c=Source[i*16+j+0x3E]^0xFF;
   Source[i*16+j+0x3E]=Source[(63-i)*16+j+0x3E]^0xFF;
   Source[(63-i)*16+j+0x3E]=c;
  }
 }
}

/* Demande un commentaire pour l'image et retourne la taille */
unsigned int GetComment(unsigned char *comment)
{
  unsigned int j;

 printf("o Your comment [32 chars max]:");
 gets(comment);

 j=strlen(comment);

 /* On teste si le commentaire n'est pas trop grand */
 if (j>32)
  j=32;

 /* Ajoute le 0 final */
 comment[j++]=0;

 return j;
}
